'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

(function (JsViewportSides) {
    JsViewportSides["ALL"] = "ALL";
    JsViewportSides["TOP"] = "TOP";
    JsViewportSides["RIGHT"] = "RIGHT";
    JsViewportSides["BOTTOM"] = "BOTTOM";
    JsViewportSides["LEFT"] = "LEFT";
})(exports.JsViewportSides || (exports.JsViewportSides = {}));
var jsViewportDefaults = {
    container: document.body || window,
    sides: [exports.JsViewportSides.ALL],
    top: 0,
    right: 0,
    bottom: 0,
    left: 0
};

/**
 * Viewport Utils
 *
 * @description A set of viewport utils
 * @author      Tony Samperi, tonysamperi.github.io
 * @version     3.0.0
 * @date        2021-06-18
 */
var jsViewportUtils = (function () {
    // IE 8 and lower fail this
    var canUseWindowDimensions = window && typeof window.innerHeight === typeof 0;
    var viewportUtils = {
        /**
         * Determines whether an element is within the viewport
         * @param  {HTMLElement}  elem       DOM Element (required)
         * @param  {JsViewportSettings}  options    Optional settings
         * @return {Boolean}            Whether the element was completely within the viewport
         */
        inViewport: function (elem, options) {
            var _a;
            if (!elem || typeof elem !== "object" || elem.nodeType !== 1) {
                throw new Error("First argument must be an element!");
            }
            var containerBoundingRect;
            var containerScrollTop;
            var containerScrollLeft;
            var elemBoundingRect;
            var isWindow;
            var scrollBarWidths = [0, 0];
            var sidesRegex = /^TOP$|^RIGHT$|^BOTTOM$|^LEFT$|^ALL$/;
            // TMP VARS
            var i;
            var result = !1;
            // Build configuration from defaults and user-provided settings and metadata
            var config = __assign(__assign({}, jsViewportDefaults), options);
            // Use the window as the container if the user specified the body or a non-element
            if (config.container === document.body || config.container.nodeType !== 1) {
                config.container = window;
            }
            // Get the element's bounding rectangle with respect to the viewport
            elemBoundingRect = elem.getBoundingClientRect();
            isWindow = (config.container === window);
            var isSideIn = (_a = {},
                _a[exports.JsViewportSides.TOP] = function () {
                    if (isWindow) {
                        return (elemBoundingRect.top >= config.top);
                    }
                    else {
                        return (elemBoundingRect.top >= containerScrollTop - (containerScrollTop - containerBoundingRect.top) + config.top);
                    }
                },
                _a[exports.JsViewportSides.RIGHT] = function () {
                    var ctScroll = isWindow ? containerScrollLeft : -scrollBarWidths[0];
                    return elemBoundingRect.right <= (containerBoundingRect.right + ctScroll - config.right);
                },
                _a[exports.JsViewportSides.BOTTOM] = function () {
                    var containerHeight = 0;
                    if (isWindow) {
                        if (canUseWindowDimensions) {
                            containerHeight = config.container.innerHeight;
                        }
                        else if (document && document.documentElement) {
                            containerHeight = document.documentElement.clientHeight;
                        }
                    }
                    else {
                        containerHeight = containerBoundingRect.bottom;
                    }
                    return (elemBoundingRect.bottom <= containerHeight - scrollBarWidths[1] - config.bottom);
                },
                _a[exports.JsViewportSides.LEFT] = function () {
                    if (isWindow) {
                        return (elemBoundingRect.left >= config.left);
                    }
                    else {
                        return (elemBoundingRect.left >= containerScrollLeft - (containerScrollLeft - containerBoundingRect.left) + config.left);
                    }
                },
                // Element is within all four boundaries
                _a[exports.JsViewportSides.ALL] = function () {
                    return (isSideIn.TOP() && isSideIn.BOTTOM() && isSideIn.LEFT() && isSideIn.RIGHT());
                },
                _a);
            // Get viewport dimensions and offsets
            if (isWindow) {
                containerBoundingRect = document.documentElement.getBoundingClientRect();
                containerScrollTop = document.body.scrollTop;
                containerScrollLeft = window.scrollX || document.body.scrollLeft;
            }
            else {
                containerBoundingRect = config.container.getBoundingClientRect();
                containerScrollTop = config.container.scrollTop;
                containerScrollLeft = config.container.scrollLeft;
            }
            if (containerScrollLeft) {
                scrollBarWidths[0] = 18;
            }
            if (containerScrollTop) {
                scrollBarWidths[1] = 16;
            }
            // Loop through all of the sides
            i = config.sides.length;
            while (i--) {
                // Test the element against each side of the viewport that was requested
                if (sidesRegex.test(config.sides[i])) {
                    if (isSideIn[config.sides[i]]()) {
                        result = !0;
                    }
                    else {
                        result = !1;
                        break;
                    }
                }
            }
            return result;
        },
        inViewportTop: function (element) {
            return viewportUtils.inViewport(element, { sides: [exports.JsViewportSides.TOP] });
        },
        inViewportRight: function (element) {
            return viewportUtils.inViewport(element, { sides: [exports.JsViewportSides.RIGHT] });
        },
        inViewportBottom: function (element) {
            return viewportUtils.inViewport(element, { sides: [exports.JsViewportSides.BOTTOM] });
        },
        inViewportLeft: function (element) {
            return viewportUtils.inViewport(element, { sides: [exports.JsViewportSides.LEFT] });
        }
    };
    return viewportUtils;
})();
var inViewport = jsViewportUtils.inViewport;
var inViewportTop = jsViewportUtils.inViewportTop;
var inViewportRight = jsViewportUtils.inViewportRight;
var inViewportBottom = jsViewportUtils.inViewportBottom;
var inViewportLeft = jsViewportUtils.inViewportLeft;

exports.inViewport = inViewport;
exports.inViewportTop = inViewportTop;
exports.inViewportRight = inViewportRight;
exports.inViewportBottom = inViewportBottom;
exports.inViewportLeft = inViewportLeft;
exports.jsViewportDefaults = jsViewportDefaults;
//# sourceMappingURL=js-viewport-utils.es5.js.map
